#include "smtlib_syntax.hh"
#include <boost/algorithm/string.hpp>

void printEscapedMathematica(std::string s, std::ostream& out) {
  boost::replace_all(s, "_", "\\[Alpha]");
  boost::replace_all(s, "$", "\\[Beta]");
  boost::replace_all(s, "?", "\\[Lambda]");
  out << s;
}

void FLet::printMathematica(std::ostream& out) const {
  out << "With[{";
  printEscapedMathematica(boundVariable, out);
  out << " = ";
  binding -> printMathematica(out);
  out << "}, ";
  subFormula -> printMathematica(out);
  out << "]";
}

void ManySubFormulas::printSubtermsMathematica(std::ostream& out) const {
  const_iterator it=begin();
  if (it == end()) return;
  (*it) -> printMathematica(out);
  it++;
  while (it != end()) {
    out << ", ";
    (*it) -> printMathematica(out);
    it++;
  }
}

void And::printMathematica(std::ostream& out) const {
  if (begin() == end()) {
    out << "True";
  } else {
    out << "And[";
    printSubtermsMathematica(out);
    out << "]";
  }
}

void Iff::printMathematica(std::ostream& out) const {
  /* TODO */
}

void Or::printMathematica(std::ostream& out) const {
  if (begin() == end()) {
    out << "False";
  } else {
    out << "Or[";
    printSubtermsMathematica(out);
    out << "]";
  }
}

void PropositionalVariable::printMathematica(std::ostream& out) const {
  printEscapedMathematica(name, out);
}

void Not::printMathematica(std::ostream& out) const {
  out << "Not[";
  f -> printMathematica(out);
  out << "]";
}

void ManyExpression::printSubtermsMathematica(std::ostream& out) const {
  const_iterator it=begin();
  if (it == end()) return;
  (*it) -> printMathematica(out);
  it++;
  while (it != end()) {
    out << ", ";
    (*it) -> printMathematica(out);
    it++;
  }
}

void UnaryNeg::printMathematica(std::ostream& out) const {
  out << "-";
  f -> printMathematica(out);
}

void NumericVariable::printMathematica(std::ostream& out) const {
  printEscapedMathematica(name, out);
}

void IntegerConstant::printMathematica(std::ostream& out) const {
  out << value;
}

void Sum::printMathematica(std::ostream& out) const {
  out << "Plus[";
  printSubtermsMathematica(out);
  out << "]";
}

void Product::printMathematica(std::ostream& out) const {
  out << "Times[";
  printSubtermsMathematica(out);
  out << "]";
}

void Quotient::printMathematica(std::ostream& out) const {
  out << "Divide[";
  e1 -> printMathematica(out);
  out << ", ";
  e2 -> printMathematica(out);
  out << "]";
}

void Equality::printMathematica(std::ostream& out) const {
  const_iterator it1=begin();
  if (it1 == end()) {
    out << "True";
    return;
  }
  const_iterator it2=it1; it2++;
  if (it2 == end()) {
    out << "True";
    return;
  }
  const_iterator it3=it2; it3++;
  if (it3 == end()) {
    out << "Equal[";
    (*it1) -> printMathematica(out);
    out << ", ";
    (*it2) -> printMathematica(out);
    out << "]";
    return;
  }
  /* TODO */
}

void PairwiseDistinct::printMathematica(std::ostream& out) const {
  const_iterator it1=begin();
  if (it1 == end()) {
    out << "True";
    return;
  }
  const_iterator it2=it1; it2++;
  if (it2 == end()) {
    out << "True";
    return;
  }
  const_iterator it3=it2; it3++;
  if (it3 == end()) {
    out << "!Equal[";
    (*it1) -> printMathematica(out);
    out << ", ";
    (*it2) -> printMathematica(out);
    out << "]";
    return;
  }
  /* TODO */
}

void Comparison::printMathematica(std::ostream& out) const {
  static const char* operators[] = { "<=", "<", ">=", ">" };

  e1 -> printMathematica(out);
  out << " " << operators[comparisonKind] << " ";
  e2 -> printMathematica(out);
}

void ExprLet::printMathematica(std::ostream& out) const {
  out << "With[{";
  printEscapedMathematica(boundVariable, out);
  out << " = ";
  binding -> printMathematica(out);
  out << "}, ";
  subFormula -> printMathematica(out);
  out << "]";
}

void FormulaIfThenElse::printMathematica(std::ostream& out) const {
  out << "If[";
  condition -> printMathematica(out);
  out << ", ";
  e1 -> printMathematica(out);
  out << ", ";
  e2 -> printMathematica(out);
  out << "]";
}

void ExpressionIfThenElse::printMathematica(std::ostream& out) const {
  out << "If[";
  condition -> printMathematica(out);
  out << ", ";
  e1 -> printMathematica(out);
  out << ", ";
  e2 -> printMathematica(out);
  out << "]";
}
