#include "integerSimplex.hh"

int main() {
  std::vector<std::string> variableNames;
  variableNames.push_back("x");
  variableNames.push_back("y");
  variableNames.push_back("delta1");
  variableNames.push_back("delta2");
  variableNames.push_back("delta3");

  
  IntegerSimplex simplex(variableNames);
  {
    LinearForm<coefficient> l(variableNames);
    l.addMonomial(0, 3);
    l.addMonomial(1, 1);
    simplex.assignBasic(2, l);
  }
  {
    LinearForm<coefficient> l(variableNames);
    l.addMonomial(0, -1);
    l.addMonomial(1, 2);
    simplex.assignBasic(3, l);
  }
  {
    LinearForm<coefficient> l(variableNames);
    l.addMonomial(0, 8);
    l.addMonomial(1, -5);
    simplex.assignBasic(4, l);
  }

  simplex.setIntegerFlag(0, true);
  simplex.setIntegerFlag(1, true);
  simplex.setIntegerFlag(2, true);
  simplex.setIntegerFlag(3, true);
  simplex.setIntegerFlag(4, true);

  simplex.assertLowerBound(2, coordinate(coefficient(3)/2));
  simplex.assertUpperBound(3, coordinate(0));
  simplex.assertUpperBound(4, coordinate(11));

  simplex.checkWellFormed();

  if (simplex.check()) {
    std::cout << "x = " << simplex.getCurrent(0) << std::endl;
    std::cout << "y = " << simplex.getCurrent(1) << std::endl;
  } else {
    std::cout << "unsat" << std::endl;
  }

  simplex.checkWellFormed();
  //std::cout << simplex;
  simplex.printStatistics(std::cout);

  return 0;
}

std::ostream& operator<<(std::ostream& out, const IntegerSimplex& s) {
  return (out << s.getRationalSimplex());
}
