#include "smtlib_parser.hh"
#include "timings.h"

int main(int argc, char **argv) {
#ifdef TIMINGS
  double parsing_time=0., compile_time=0., solve_time=0.;
  TIMING_DECLARE();
#endif

  for(unsigned index=1; index<argc; index++) {
  TIMING_BEGINS();
  smtlib_parser parser;
  smtlib_in = fopen(argv[index], "r");
  parser.parse_benchmark();
  TIMING_ENDS(parsing_time);

  if (parser.formula==0) {
    std::cerr << "bad formula" << std::endl;
    exit(1);
  }

  TIMING_BEGINS();
  FormulaCompiler solver;
  Environment env;

  {
    identifierSet numVars;
    parser.formula->collectNumericVariables(numVars);
    for(smtlib_parser::assumptions_t::const_iterator
	  it=parser.assumptions.begin();
	it != parser.assumptions.end(); it++) {
      (*it)->collectNumericVariables(numVars);
    }
	
    for(identifierSet::const_iterator it=numVars.begin();
	it!=numVars.end();
	it++) {
      solver.getNumVar(*it);
    }
  }

  vec<Lit> booleanAssumptions;
  booleanAssumptions.push(parser.formula->compile(solver, env));
  for(smtlib_parser::assumptions_t::const_iterator
	it=parser.assumptions.begin();
      it != parser.assumptions.end(); it++) {
    booleanAssumptions.push((*it)->compile(solver, env));
  }
  TIMING_ENDS(compile_time);

  TIMING_BEGINS();
  bool solvable = solver.solve(booleanAssumptions);
  TIMING_ENDS(solve_time);

  std::cout << "VERDICT: " << (solvable ? "sat" : "unsat") << std::endl;
  if ((parser.status == SAT && !solvable)
      || (parser.status == UNSAT && solvable)) {
    std::cerr << "BAD ANSWER" << std::endl;
  }

#ifdef TIMINGS
  std::cout << "parsing: " << parsing_time
	    << " compile: " << compile_time
	    << " solve: " << solve_time << std::endl;
#endif

#ifdef PRINT_PROPMODEL
  std::cout << "targets:";
  for(unsigned i=0; i<booleanAssumptions.size(); i++) {
    std::cout << " " << booleanAssumptions[i];
  }
  std::cout << std::endl;

  std::vector<Proposition> propMeanings = solver.propMeanings();
  for(unsigned i=0; i<propMeanings.size(); i++) {
    const Proposition& prop = propMeanings[i];
    std::cout << "#" << i << ": " << prop;
    if (solvable) {
      std::cout << " = " << solver.modelValue(i);
    }
    std::cout << std::endl;
  }
#endif
  }
  return 0;
}
