#include "smtlib_syntax.hh"
#include <iostream>
#include <sstream>

Formula::~Formula() {
}

void Formula::addToConjunction(And& conjunction) {
  conjunction.addFormulaWithoutLooking(this);
}

void Formula::addToDisjunction(Or& disjunction) {
  disjunction.addFormulaWithoutLooking(this);
}

Expression::~Expression() {
}

void Formula::printIdentifier(std::ostream& o) {
  o << "@" << (void*) this;
}

void Comparison::collectNumericVariables(identifierSet& s) const {
  e1->collectNumericVariables(s);
  e2->collectNumericVariables(s);
}

void Comparison::collectPropositionalVariables(identifierSet& s) const {
  e1->collectPropositionalVariables(s);
  e2->collectPropositionalVariables(s);
}

void PropositionalVariable::collectPropositionalVariables(identifierSet& s) const {
  s.insert(name);
}

void PropositionalVariable::collectNumericVariables(identifierSet& s) const {
}

void ManyExpression::collectPropositionalVariables(identifierSet& s) const {
  for(const_iterator i=begin(); i!=end(); ++i) {
    (*i)->collectPropositionalVariables(s);
  }
}

void ManyExpression::collectNumericVariables(identifierSet& s) const {
  for(const_iterator i=begin(); i!=end(); ++i) {
    (*i)->collectNumericVariables(s);
  }
}

void ManySubFormulas::collectPropositionalVariables(identifierSet& s) const {
  for(const_iterator i=begin(); i!=end(); ++i) {
    (*i)->collectPropositionalVariables(s);
  }
}

void ManySubFormulas::collectNumericVariables(identifierSet& s) const {
  for(const_iterator i=begin(); i!=end(); ++i) {
    (*i)->collectNumericVariables(s);
  }
}

int ManySubFormulas::size()  const {
  int total=1;
  for(const_iterator i = subFormulas.begin();
      i != subFormulas.end();
      ++i) {
    total += (*i)->size();
  }
  return total;
}

ManySubFormulas::~ManySubFormulas() {
  for(const_iterator i = subFormulas.begin();
      i != subFormulas.end();
      ++i) {
    delete(*i);
  }
}

void ManySubFormulas::printSubFormulas(std::ostream& o) const {
  for(const_iterator i = begin(); i != end(); ++i) {
    o << " " << **i;
  }
}

#ifdef COLLAPSE_COMBINATORS
void And::addToConjunction(And& conjunction) {
  for(iterator it=begin(); it!=end(); it++) {
    (*it)->addToConjunction(conjunction);
  }
  subFormulas.clear();
  delete this;
}

void Or::addToDisjunction(Or& disjunction) {
  for(iterator it=begin(); it!=end(); it++) {
    (*it)->addToDisjunction(disjunction);
  }
  subFormulas.clear();
  delete this;
}
#endif

void And::print(std::ostream& o) const {
  o << "(and";
  printSubFormulas(o);
  o << ")";
}

void Iff::print(std::ostream& o) const {
  o << "(iff";
  printSubFormulas(o);
  o << ")";
}

void Or::print(std::ostream& o) const {

  o << "(or";
  printSubFormulas(o);
  o << ")";
}

void PropositionalVariable::print(std::ostream& o) const {
  o << name;
}

void Not::print(std::ostream& o) const {
  o << "(not " << *f << ")";
}

integer mpz_one(1), mpz_zero(0);

std::ostream& operator<<(std::ostream& o, Comparison::ComparisonType cmp)
{
  switch(cmp) {
  case Comparison::LESS:
    o << "<";
    break;
  case Comparison::GREATER:
    o << ">";
    break;
  case Comparison::LESS_EQ:
    o << "<=";
    break;
  case Comparison::GREATER_EQ:
    o << ">=";
    break;
  }
  return o;
}

void Comparison::print(std::ostream& o) const {
  o << "(" << comparisonKind << " " << *e1 << " " << *e2 << ")";
}

void FLet::collectPropositionalVariables(identifierSet& s) const {
  bool wasAlreadyThere = s.count(boundVariable);
  subFormula->collectPropositionalVariables(s);
  if (!wasAlreadyThere) s.erase(boundVariable);
}

void FLet::collectNumericVariables(identifierSet& s) const {
  binding -> collectNumericVariables(s);
  subFormula->collectNumericVariables(s);
}

int FLet::size() const {
  return 1 + subFormula->size() + binding -> size();
}

void FLet::print(std::ostream& o) const {
  o << "(flet (" << boundVariable << " " << *binding << ") " << *subFormula << ")";
}

ManyExpression::~ManyExpression() {
  for(const_iterator i=begin(); i!=end(); ++i) {
    delete *i;
  }
}

void ManyExpressionExpression::printCompoundExpression(std::ostream& o, std::string prefix) const {
  o << "(" << prefix;
  for(const_iterator i=begin(); i!=end(); ++i) {
    o << " " << **i;
  }
  o << ")";
}

void UnaryNeg::print(std::ostream& o) const {
  o << "(~ " << *f << ")";
}

void NumericVariable::print(std::ostream& o) const {
  o << name;
}

void IntegerConstant::print(std::ostream& o) const {
  o << value;
}

void Sum::print(std::ostream& o) const {
  printCompoundExpression(o, "+");
}

void Product::print(std::ostream& o) const {
  printCompoundExpression(o, "*");
}

void ManyExpressionFormula::printListPredicate(std::ostream& o, std::string prefix) const {
  o << "(" << prefix;
  for(const_iterator i=begin(); i!=end(); i++) {
    o << " " << **i;
  }  
  o << ")";
}

void PairwiseDistinct::print(std::ostream& o) const {
  printListPredicate(o, "distinct");
}

void Equality::print(std::ostream& o) const {
  printListPredicate(o, "=");
}
int ExprLet::size() const {
  return 1+ subFormula->size();
}

void ExprLet::collectPropositionalVariables(identifierSet& s) const {
  subFormula->collectPropositionalVariables(s);
}

void ExprLet::collectNumericVariables(identifierSet& s) const {
  bool wasAlreadyThere = s.count(boundVariable);
  subFormula->collectNumericVariables(s);
  if (!wasAlreadyThere) s.erase(boundVariable);
  binding->collectNumericVariables(s);
}

void ExprLet::print(std::ostream& o) const {
  o << "(let (" << boundVariable << " " << *binding << ")" << *subFormula << ")";
}

int FormulaIfThenElse::size() const {
  return condition->size() + e1->size() + e2->size();
}

FormulaIfThenElse::~FormulaIfThenElse() {
  delete condition;
  delete e1;
  delete e2;
}

void FormulaIfThenElse::collectPropositionalVariables(identifierSet& s) const {
  condition -> collectPropositionalVariables(s);
  e1 -> collectPropositionalVariables(s);
  e2 -> collectPropositionalVariables(s);
}

void FormulaIfThenElse::collectNumericVariables(identifierSet& s) const {
  condition -> collectNumericVariables(s);
  e1 -> collectNumericVariables(s);
  e2 -> collectNumericVariables(s);
}

void FormulaIfThenElse::print(std::ostream& o) const {
  o << "(if_then_else " << *condition << " " << *e1 << " " << *e2 << ")";
}

void NumericVariable::collectPropositionalVariables(identifierSet& s) const {
}

void NumericVariable::collectNumericVariables(identifierSet& s) const {
  s.insert(name);
}

void IntegerConstant::collectPropositionalVariables(identifierSet& s) const {
}

void IntegerConstant::collectNumericVariables(identifierSet& s) const {
}

void UnaryNeg::collectPropositionalVariables(identifierSet& s) const {
  f -> collectPropositionalVariables(s);
}

void UnaryNeg::collectNumericVariables(identifierSet& s) const {
  f -> collectNumericVariables(s);
}

void Not::collectPropositionalVariables(identifierSet& s) const {
  f -> collectPropositionalVariables(s);
}

void Not::collectNumericVariables(identifierSet& s) const {
  f -> collectNumericVariables(s);
}

void Quotient::collectPropositionalVariables(identifierSet& s) const {
  e1 -> collectPropositionalVariables(s);
  e2 -> collectPropositionalVariables(s);
}

void Quotient::collectNumericVariables(identifierSet& s) const {
  e1 -> collectNumericVariables(s);
  e2 -> collectNumericVariables(s);
}

void Quotient::print(std::ostream& o) const {
  o << "(/ " << *e1 << " " << *e2 << ")";
}


ExpressionIfThenElse::~ExpressionIfThenElse() {
  delete condition;
  delete e1;
  delete e2;
}

void ExpressionIfThenElse::collectPropositionalVariables(identifierSet& s) const {
  condition -> collectPropositionalVariables(s);
  e1 -> collectPropositionalVariables(s);
  e2 -> collectPropositionalVariables(s);
}

void ExpressionIfThenElse::collectNumericVariables(identifierSet& s) const {
  //std::cout << "coincoin" << std::endl;
  condition -> collectNumericVariables(s);
  e1 -> collectNumericVariables(s);
  e2 -> collectNumericVariables(s);
  //std::cout << "creating " << variableName() << std::endl;
  s.insert(variableName());
}

void ExpressionIfThenElse::print(std::ostream& o) const {
  o << "(ite " << *condition << " " << *e1 << " " << *e2 << ")";
}

std::string ExpressionIfThenElse::variableName() const {
  std::stringstream name;
  name << "#@ite_var@" << this;
  return name.str();
}
